from pathlib import Path
from typing import Iterable

import watchfiles

from .compile import compile_one


def on_events(events: Iterable[watchfiles.main.FileChange], encoding: str):
    for change, name in events:
        if name.endswith(".py"):
            break

        print(change.raw_str(), name)

        if change is change.deleted:
            Path(name).with_suffix(".py").unlink(missing_ok=True)
        else:
            compile_one(name, encoding=encoding)


def watch(*paths: str | Path, encoding="utf-8", **kwargs):
    for events in watchfiles.watch(*paths, **kwargs):
        on_events(events, encoding)


async def awatch(*paths: str | Path, encoding="utf-8", **kwargs):
    async for events in watchfiles.awatch(*paths, **kwargs):
        on_events(events, encoding)


def main():
    from argparse import ArgumentParser
    from contextlib import suppress

    parser = ArgumentParser()

    parser.add_argument("paths", nargs="*", type=Path, default=[Path.cwd()])
    parser.add_argument("--encoding", "-e", default="utf-8")

    args = parser.parse_args()

    assert args.paths

    with suppress(KeyboardInterrupt):
        watch(*args.paths, encoding=args.encoding)



from pathlib import Path

from promplate.prompt.template import Template, TemplateCore

meta_template = Template.read(Path(__file__) / "../template.j2")


def template_to_code(template: TemplateCore):
    return meta_template.render({"template": template})



from pathlib import Path

from promplate import Template

from .._core import template_to_code


def compile_one(file: str | Path, *, encoding="utf-8"):
    Path(file).with_suffix(".py").write_text(template_to_code(Template.read(file, encoding=encoding)), encoding=encoding)


def compile_all(folder: str | Path, pattern="*[!.py]", *, encoding="utf-8"):
    for file in Path(folder).rglob(pattern):
        compile_one(file, encoding=encoding)
